<?php

Yii::import('application.extensions.AutoCountSqlDataProvider');

/**
 * SqlTableView is a widget based on AutoCountGridView that displays the rows of
 * a table efficiently.
 */
class SqlTableView extends CWidget
{
	/**
	 * The name of table to browse.
	 */
	public $table;
	
	/**
	 * An array of criteria to apply when fetching rows from the table.
	 */
	public $criteria;
	
	/**
	 * An associative array of functions to apply before displaying column data.
	 */
	public $formatters = array();
	
	/**
	 * The name of the GET variable used to store filter values entered by the
	 * user.
	 */
	public $filter_get_variable = 'GenericModel';
	
	/**
	 * The name of the GET variable used to force counting of the table's rows.
	 */	 
	public $count_get_variable;

	/**
	 * Display the widget.
	 */
	public function run()
	{
		$column_names = array_keys(Yii::app()->getDb()->getSchema()->getTable($this->table)->columns);
		eval('class '.$this->filter_get_variable.' extends GenericModelBase { }');
		$filter_model = new $this->filter_get_variable($column_names);
		if(isset($_GET[$this->filter_get_variable]))
		{
			$filter_model->attributes = $_GET[$this->filter_get_variable];
		}
		if (empty($this->criteria))
		{
			$criteria = new CDbCriteria();
		}
		else
		{
			$criteria = new CDbCriteria($this->criteria);
		}
		foreach ($filter_model->attributes as $key=>$value)
		{
			if (empty($value))
				continue;
			$criteria->addColumnCondition(array($key=>$value));
		}

		$column_specs = array();
		foreach ($column_names as $column)
		{
			if (isset($this->formatters[$column]))
			{
				$column_specs[] = array(
					'class'=>'CDataColumn',
					'name'=>$column,
					'value'=>$this->formatters[$column].'($data[\''.$column.'\'])',
					'header'=>$column,
					'type'=>'raw',
					'sortable'=>true,
				);
			}
			else
			{
				$column_specs[] = $column;
			}
		}
		
		$params = array(
			'dataProvider'=>new AutoCountSqlDataProvider(
				$this->table,
				$criteria,
				isset($_GET[$this->count_get_variable]),
				array(
					'sort'=>array(
						'attributes'=>$column_names,
					),
					'pagination'=>array(
						'pageSize'=>10,
					),
				)
			),
			'filter'=>$filter_model,
			'columns'=>$column_specs,
			'selectableRows'=>0,
			'template'=>"{summary}\n<div style='overflow:auto'>{items}</div>\n{pager}",
		);
		if (!empty($this->count_get_variable))
		{
			$params['count_get_variable'] = $this->count_get_variable;
		}
		$this->widget('application.extensions.AutoCountGridView', $params);
	}
}

/**
 * A generic model used to provide filtering capabilities to the widget.
 */
class GenericModelBase
{
	public $attributes = array();

	public function __construct($attr_names)
	{
		foreach ($attr_names as $attr_name)
		{
			$this->attributes[$attr_name] = '';
		}
	}

	public function __get($name)
	{
		return $this->attributes[$name];
	}

	public function __set($name, $value)
	{
		$this->attributes[$name] = $value;
	}

	public function getValidators()
	{
		return array();
	}
	
	public function hasErrors()
	{
		return false;
	}
}
